// Copyright 2012-2014, Philip Quinn. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//  TSLoggingResponder.m
//  Touch Scroll Simulator
//
//  Created by Philip Quinn on 25/05/12.
//  Copyright (c) 2012 Philip Quinn. All rights reserved.
//

#import "TSLoggingResponder.h"
#import <objc/runtime.h>
#import <objc/message.h>
#import "TSLogger.h"

static void ts_touchesBegan(id self, SEL _cmd, NSSet *touches, UIEvent *event)
{
    struct objc_super superclass = { self, class_getSuperclass(object_getClass(self)) };
    
    NSString *touchStr = TSLogStringFromTouches(touches);
    [[TSLogger sharedLogger] writeLineFormat:@"TOUCH#DOWN%@", touchStr];
    
    objc_msgSendSuper(&superclass, _cmd, touches, event);
}

static void ts_touchesMoved(id self, SEL _cmd, NSSet *touches, UIEvent *event)
{
    struct objc_super superclass = { self, class_getSuperclass(object_getClass(self)) };
    
    NSString *touchStr = TSLogStringFromTouches(touches);
    [[TSLogger sharedLogger] writeLineFormat:@"TOUCH#MOVE%@", touchStr];
    
    objc_msgSendSuper(&superclass, _cmd, touches, event);
}

static void ts_touchesEnded(id self, SEL _cmd, NSSet *touches, UIEvent *event)
{
    struct objc_super superclass = { self, class_getSuperclass(object_getClass(self)) };
    
    NSString *touchStr = TSLogStringFromTouches(touches);
    [[TSLogger sharedLogger] writeLineFormat:@"TOUCH#UP%@", touchStr];
    
    objc_msgSendSuper(&superclass, _cmd, touches, event);
}

static void ts_touchesCancelled(id self, SEL _cmd, NSSet *touches, UIEvent *event)
{
    struct objc_super superclass = { self, class_getSuperclass(object_getClass(self)) };
    
    NSString *touchStr = TSLogStringFromTouches(touches);
    [[TSLogger sharedLogger] writeLineFormat:@"TOUCH#CANCEL%@", touchStr];
    
    objc_msgSendSuper(&superclass, _cmd, touches, event);
}

Class TSLoggingScrollViewPanGestureRecognizer()
{
    static Class klass = NULL;
    if (klass) return klass;
    
    Class superclass = NSClassFromString(@"UIScrollViewPanGestureRecognizer");
    klass = objc_allocateClassPair(superclass, "TSLoggingScrollViewPanGestureRecognizer", 0);
    
    class_addMethod(klass, @selector(touchesBegan:withEvent:),
                    (IMP)ts_touchesBegan,
                    "v@:@@");
    class_addMethod(klass, @selector(touchesMoved:withEvent:),
                    (IMP)ts_touchesMoved,
                    "v@:@@");
    class_addMethod(klass, @selector(touchesEnded:withEvent:),
                    (IMP)ts_touchesEnded,
                    "v@:@@");
    class_addMethod(klass, @selector(touchesCancelled:withEvent:),
                    (IMP)ts_touchesCancelled,
                    "v@:@@");

    objc_registerClassPair(klass);
    return klass;
}
